#
#  Name:    Mp.py
#
#  Purpose: Python module interface to MX.
#
#  Author:  William Lavender
#
#-----------------------------------------------------------------------
#
#  Copyright 2002-2004 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

import re

import MpPrivate

#
#  Initialize the Mp package
#

MpPrivate.init()

#=============================================================================
#
#  Mp Module functions
#

def setup_database( filename ):
	#
	#  Create a new record list.
	#

	record_list = RecordList()

	#
	#  Enable plotting
	#

	record_list.set_plot_enable(1)

	#
	#  Read the database in and initialize the corresponding hardware.
	#

	record_list.read_database( filename )

	record_list.finish_database_initialization()

	record_list.initialize_hardware( 0 )

	record_list.unsaved_scans = 0

	return record_list

#
#  Mp output functions
#

def info( message_string ):
	MpPrivate.info( message_string )

def set_info_output_function( function ):
	MpPrivate.set_info_output_function( function )

###

def warning( message_string ):
	MpPrivate.warning( message_string )

def set_warning_output_function( function ):
	MpPrivate.set_warning_output_function( function )

###

def debug( debug_level, message_string ):
	MpPrivate.debug( debug_level, message_string )

def set_debug_output_function( function ):
	MpPrivate.set_debug_output_function( function )

###

def set_error_output_function( function ):
	MpPrivate.set_error_output_function( function )

###

def info_dialog( text_prompt, gui_prompt, button_label ):
	MpPrivate.info_dialog( text_prompt, gui_prompt, button_label )

def set_info_dialog_function( function ):
	MpPrivate.set_info_dialog_function( function )

#
#  Mp user interrupt functions
#

def getch():
	return MpPrivate.getch()

def kbhit():
	return MpPrivate.kbhit()

def user_requested_interrupt():
	return MpPrivate.user_requested_interrupt()

def set_user_interrupt_function( function ):
	MpPrivate.set_user_interrupt_function( function )

#
#  Miscellaneous module functions
#

def get_debug_level():
	return MpPrivate.get_debug_level()

def set_debug_level( debug_level ):
	MpPrivate.set_debug_level( debug_level )

def get_version():
	return MpPrivate.get_version()

def construct_record_object( handle ):
	record_type_string = MpPrivate.get_object_class_name( handle )

	command = "result = %s()" % record_type_string

	exec command

	return result

#=============================================================================
#
#  Mp Classes
#

#
#  RecordList class
#

class RecordList:
	def __init__( self ):
		self.unsaved_scans = 0

		# Create a C record list and get the long integer handle that
		# corresponds to the C list head record.

		list_head_handle = MpPrivate.create_record_list()

		list_head_record = construct_record_object( list_head_handle )

		list_head_record.handle = list_head_handle

		list_head_record.name = \
				MpPrivate.get_record_name( list_head_handle )

		# Store copies of the Python list head object in the Python
		# Mp.RecordList object.

		self.list_head_handle = list_head_handle

		self.list_head_record = list_head_record

	def read_database( self, filename ):

		MpPrivate.read_database( self.list_head_handle, filename )

		# Create a shadow linked list of Mp.Record objects
		# that parallels the MX_RECORD linked list in the
		# C data structures.  This is inserted after the
		# last Mp.Record that already exists in the Python
		# database.

		list_head_record = self.list_head_record
		list_head_handle = self.list_head_handle

		current_record = list_head_record.previous_record
		current_handle = current_record.handle

		while ( 1 ):

			# Find the handle corresponding to the next MX_RECORD
			# in the C database.

			next_handle = \
				MpPrivate.get_next_record( current_handle )

			if ( next_handle == list_head_handle ):
				# We have completed the loop and are back
				# at the start of the C record list.  This
				# means we are done and can now exit the
				# while loop.

				break		

			# Create and fill in a Python Mp.Record object to match
			# the C MX_RECORD.

			next_record = construct_record_object( next_handle )

			next_record.handle = next_handle

			next_record.previous_record = current_record
			next_record.next_record     = list_head_record

			next_record.name = \
				MpPrivate.get_record_name( next_handle )

			# Insert the Python Mp.Record in the Python record list.

			list_head_record.previous_record = next_record
			current_record.next_record = next_record

			current_record = next_record
			current_handle = next_handle

	def finish_database_initialization( self ):
		MpPrivate.finish_database_initialization( \
						self.list_head_handle )

	def initialize_hardware( self, inithw_flags ):
		MpPrivate.initialize_hardware( self.list_head_handle, \
						inithw_flags )

	def shutdown_hardware( self ):
		MpPrivate.shutdown_hardware( self.list_head_handle )

	def create_record_from_description( self, description ):
		# Create a C MX_RECORD and get back a handle for it.

		handle = MpPrivate.create_record_from_description( \
					self.list_head_handle, description )

		# Construct a Python Mp.Record that corresponds to it.

		record = construct_record_object( handle )

		record.handle = handle
		record.name = MpPrivate.get_record_name( handle )

		# The C database generally adds records to the end of the
		# circular MX_RECORD list, so we should prepare to do the same.

		list_head_record = self.list_head_record
		previous_record  = list_head_record.previous_record

		record.next_record     = list_head_record
		record.previous_record = previous_record

		# Now insert the Mp.Record into the circular list.

		list_head_record.previous_record = record
		previous_record.next_record      = record

		# Return the record we have just created.

		return record

	def get_record( self, record_name ):

		list_head_record = self.list_head_record

		if ( list_head_record.name == record_name ):
			return list_head_record

		current_record = list_head_record.next_record

		# Search through the record list looking for the requested
		# record.

		while ( current_record != list_head_record ):

			# If we find it, then return.

			if ( current_record.name == record_name ):
				return current_record

			current_record = current_record.next_record

		# If we get here, then we did not find the record
		# and must return an error.

		error_string = \
			"The record '%s' does not exist." % ( record_name )

		raise ValueError, error_string

	def get_multiple_records( self, mx_superclass, mx_class, mx_type, \
					reverse_order ):

		if ( reverse_order == 0 ):
			current_record = self.list_head_record.next_record
		else:
			current_record = self.list_head_record.previous_record

		python_record_list = []

		while ( current_record != self.list_head_record ):

			current_name = current_record.name

			current_superclass = \
				current_record.get_field( "mx_superclass" )

			match = re.search( mx_superclass, current_superclass )

			if match:
				current_class = \
					current_record.get_field( "mx_class" )

				match = re.search( mx_class, current_class )

				if match:
					current_type = \
					  current_record.get_field( "mx_type" )

					match = re.search( mx_type, \
								current_type )
					if match:
						python_record_list.append( \
								current_name )

			if ( reverse_order == 0 ):
				current_record = current_record.next_record
			else:
				current_record = current_record.previous_record

		return python_record_list

	def plotting_is_enabled( self ):
		return MpPrivate.plotting_is_enabled( self.list_head_handle )

	def set_plot_enable( self, plot_enable_value ):
		MpPrivate.set_plot_enable( self.list_head_handle, \
						plot_enable_value )

	def set_program_name( self, program_name ):
		MpPrivate.set_program_name( self.list_head_handle, \
						program_name )

	def log_open( self ):
		MpPrivate.log_open( self.list_head_handle )

	def log_close( self ):
		MpPrivate.log_close( self.list_head_handle )

#
#  Record class
#

class Record:
	def __init__( self ):
		self.previous_record = self
		self.next_record = self

	def create_description_from_record( self ):
		return MpPrivate.create_description_from_record( self.handle )

	def finish_record_initialization( self ):
		MpPrivate.finish_record_initialization( self.handle )

	def delete_record( self ):
		previous_record = self.previous_record
		next_record = self.next_record

		# Remove this record from the Python record list.

		next_record.previous_record = previous_record
		previous_record.next_record = next_record

		# Then, delete the C record.

		MpPrivate.delete_record( self.handle )

		# Finally, delete the Python record.

		del self

	def get_next_record( self ):
		return self.next_record

	def get_previous_record( self ):
		return self.previous_record

	def get_field( self, field_name ):
		return MpPrivate.get_field( self.handle, field_name )

	def set_field( self, field_name, string_value ):
		MpPrivate.set_field( self.handle, field_name, string_value )

#
#  CAMAC interface class
#

class CAMAC(Record):
	def command( self, slot, subaddress, function_code, data, \
								mode="normal" ):
		return MpPrivate.camac_command( self.handle, \
				slot, subaddress, function_code, data, mode )

	def get_lam_status( self ):
		return MpPrivate.camac_get_lam_status( self.handle )

	def controller_command( self, command ):
		MpPrivate.camac_controller_command( self.handle, command )

#
#  GPIB interface class
#

class GPIB(Record):
	def getline( self, address, maximum_length=1024 ):
		return MpPrivate.gpib_getline( self.handle, \
						address, maximum_length )

	def putline( self, address, line ):
		MpPrivate.gpib_putline( self.handle, address, line )

	def interface_clear( self ):
		MpPrivate.gpib_interface_clear( self.handle )

	def device_clear( self ):
		MpPrivate.gpib_device_clear( self.handle )

	def selective_device_clear( self, address ):
		MpPrivate.gpib_selective_device_clear( self.handle, address )

	def local_lockout( self ):
		MpPrivate.gpib_local_lockout( self.handle )

	def remote_enable( self, address ):
		MpPrivate.gpib_remote_enable( self.handle, address )

	def go_to_local( self, address ):
		MpPrivate.gpib_go_to_local( self.handle, address )

	def trigger( self, address ):
		MpPrivate.gpib_trigger( self.handle, address )

	def wait_for_service_request( self, timeout ):
		MpPrivate.gpib_wait_for_service_request( self.handle, timeout )

	def serial_poll( self, address ):
		return MpPrivate.gpib_serial_poll( self.handle, address )

	def serial_poll_disable( self ):
		MpPrivate.gpib_serial_poll_disable( self.handle )

#
#  MODBUS interface class
#

class MODBUS(Record):
	def read_coils( self, starting_address, num_coils ):
		return MpPrivate.modbus_read_coils( self.handle, \
					starting_address, num_coils )

	def read_discrete_inputs( self, starting_address, num_inputs ):
		return MpPrivate.modbus_read_discrete_inputs( self.handle, \
					starting_address, num_inputs )

	def read_holding_registers( self, starting_address, num_registers ):
		return MpPrivate.modbus_read_holding_registers( self.handle, \
					starting_address, num_registers )

	def read_input_registers( self, starting_address, num_registers ):
		return MpPrivate.modbus_read_input_registers( self.handle, \
					starting_address, num_registers )

	def write_single_coil( self, coil_address, coil_value ):
		MpPrivate.modbus_write_single_coil( self.handle, \
					coil_address, coil_value )

	def write_single_register( self, register_address, register_value ):
		MpPrivate.modbus_write_single_register( self.handle, \
					register_address, register_value )

	def read_exception_status( self ):
		return MpPrivate.modbus_read_exception_status( self.handle )

	def get_comm_event_counter( self ):
		return MpPrivate.modbus_get_comm_event_counter( self.handle )

	def write_multiple_coils( self, starting_address, coil_value_tuple ):
		MpPrivate.modbus_write_multiple_coils( self.handle, \
					starting_address, coil_value_tuple )

	def write_multiple_registers( self, \
				starting_address, register_value_tuple ):
		MpPrivate.modbus_write_multiple_registers( self.handle, \
				starting_address, register_value_tuple )

	def mask_write_register( self, register_address, and_mask, or_mask ):
		MpPrivate.modbus_mask_write_register( self.handle, \
				register_address, and_mask, or_mask )

	def read_write_multiple_registers( self, \
			read_starting_address, num_registers_to_read, \
			write_starting_address, write_register_tuple ):
		return MpPrivate.modbus_read_write_multiple_registers( \
			self.handle, \
			read_starting_address, num_registers_to_read, \
			write_starting_address, write_register_tuple )

	def read_fifo_queue( self, fifo_pointer_address, max_fifo_values ):
		return MpPrivate.modbus_read_fifo_queue( self.handle, \
			fifo_pointer_address, max_fifo_values )
#
#  Port I/O interface class
#

class Portio(Record):
	def inp8( self, port_number ):
		return MpPrivate.portio_inp8( self.handle, port_number )

	def inp16( self, port_number ):
		return MpPrivate.portio_inp16( self.handle, port_number )

	def outp8( self, port_number, value ):
		MpPrivate.portio_outp8( self.handle, port_number, value )

	def outp16( self, port_number, value ):
		MpPrivate.portio_outp16( self.handle, port_number, value )

#
#  RS-232 interface class
#

class RS232(Record):
	def getchar( self ):
		return MpPrivate.rs232_getchar( self.handle )

	def putchar( self, c ):
		MpPrivate.rs232_putchar( self.handle, c )

	def read( self, num_bytes ):
		return MpPrivate.rs232_read( self.handle, num_bytes )

	def write( self, buffer ):
		MpPrivate.rs232_write( self.handle, buffer )

	def getline( self, maximum_length=1024 ):
		return MpPrivate.rs232_getline( self.handle, maximum_length )

	def putline( self, line ):
		MpPrivate.rs232_putline( self.handle, line )

	def input_is_available( self ):
		return MpPrivate.rs232_input_is_available( self.handle )

	def discard_unread_input( self ):
		MpPrivate.rs232_discard_unread_input( self.handle )

	def discard_unwritten_output( self ):
		MpPrivate.rs232_discard_unwritten_output( self.handle )

	def get_signal_state( self ):
		return MpPrivate.rs232_get_signal_state( self.handle )

	def set_signal_state( self, signal_state ):
		MpPrivate.rs232_set_signal_state( self.handle, signal_state )

#
#  USB interface class
#

class USB(Record):
	def enumerate_devices( self ):
		MpPrivate.usb_enumerate_devices( self.handle )

	def find_device_by_order( self, vendor_id, product_id, order_number, \
			configuration_number, interface_number, \
			alternate_interface_number, quiet_flag ):

		device_handle = MpPrivate.usb_find_device_by_order( \
			self.handle, vendor_id, product_id, order_number, \
			configuration_number, interface_number, \
			alternate_interface_number, quiet_flag )

		return USBDevice( self, device_handle )

	def find_device_by_serial_number( self, \
			vendor_id, product_id, serial_number, \
			configuration_number, interface_number, \
			alternate_interface_number, quiet_flag ):

		device_handle = MpPrivate.usb_find_device_by_order( \
			self.handle, vendor_id, product_id, serial_number, \
			configuration_number, interface_number, \
			alternate_interface_number, quiet_flag )

		return USBDevice( self, device_handle )

#
#  USBDevice class
#

class USBDevice:
	def __init__( self, bus, handle ):
		self.bus = bus
		self.handle = handle

	def delete_device( self ):
		MpPrivate.usb_delete_device( self.bus.handle, self.handle )

	def reset_device( self ):
		MpPrivate.usb_reset_device( self.bus.handle, self.handle )

	def control_transfer( self, \
			request_type, request, value, \
			index_or_offset, write_buffer, \
			num_bytes_to_read, timeout ):

		return MpPrivate.usb_control_transfer( \
			self.bus.handle, self.handle, \
			request_type, request, value, \
			index_or_offset, write_buffer, \
			num_bytes_to_read, timeout )

	def bulk_read( self, endpoint_number, num_bytes_to_read, timeout ):

		return MpPrivate.usb_bulk_read( self.bus.handle, self.handle, \
			endpoint_number, num_bytes_to_read, timeout )

	def bulk_write( self, endpoint_number, write_buffer, timeout ):

		MpPrivate.usb_bulk_write( self.bus.handle, self.handle, \
			endpoint_number, write_buffer, timeout )

#
#  VME interface class
#

class VME(Record):
	def in8( self, crate, address_mode, address ):
		return MpPrivate.vme_in8( self.handle, \
				crate, address_mode, address )

	def in16( self, crate, address_mode, address ):
		return MpPrivate.vme_in16( self.handle, \
				crate, address_mode, address )

	def in32( self, crate, address_mode, address ):
		return MpPrivate.vme_in32( self.handle, \
				crate, address_mode, address )

	def out8( self, crate, address_mode, address, value ):
		MpPrivate.vme_out8( self.handle, \
				crate, address_mode, address, value )

	def out16( self, crate, address_mode, address, value ):
		MpPrivate.vme_out16( self.handle, \
				crate, address_mode, address, value )

	def out32( self, crate, address_mode, address, value ):
		MpPrivate.vme_out32( self.handle, \
				crate, address_mode, address, value )

	def multi_in8( self, crate, address_mode, address_increment, \
			address, num_values ):
		return MpPrivate.vme_multi_in8( self.handle, crate, \
			address_mode, address_increment, address, num_values )

	def multi_in16( self, crate, address_mode, address_increment, \
			address, num_values ):
		return MpPrivate.vme_multi_in16( self.handle, crate, \
			address_mode, address_increment, address, num_values )

	def multi_in32( self, crate, address_mode, address_increment, \
			address, num_values ):
		return MpPrivate.vme_multi_in32( self.handle, crate, \
			address_mode, address_increment, address, num_values )

	def multi_out8( self, crate, address_mode, address_increment, \
			address, values_tuple ):
		MpPrivate.vme_multi_out8( self.handle, crate, \
			address_mode, address_increment, address, values_tuple )

	def multi_out16( self, crate, address_mode, address_increment, \
			address, values_tuple ):
		MpPrivate.vme_multi_out16( self.handle, crate, \
			address_mode, address_increment, address, values_tuple )

	def multi_out32( self, crate, address_mode, address_increment, \
			address, values_tuple ):
		MpPrivate.vme_multi_out32( self.handle, crate, 
			address_mode, address_increment, address, values_tuple )

	def get_read_address_increment( self, crate ):
		return MpPrivate.vme_get_read_address_increment( self.handle, \
									crate )

	def set_read_address_increment( self, crate, address_increment ):
		MpPrivate.vme_set_read_address_increment( self.handle, crate, \
							address_increment )

	def get_write_address_increment( self, crate ):
		return MpPrivate.vme_get_write_address_increment( self.handle, \
									crate )

	def set_write_address_increment( self, crate, address_increment ):
		MpPrivate.vme_set_write_address_increment( self.handle, crate, \
							address_increment )

#
#  Amplifier class
#

class Amplifier(Record):

	def get_gain( self ):
		return MpPrivate.amplifier_get_gain( self.handle )

	def set_gain( self, value ):
		MpPrivate.amplifier_set_gain( self.handle, value )

	def get_offset( self ):
		return MpPrivate.amplifier_get_offset( self.handle )

	def set_offset( self, value ):
		MpPrivate.amplifier_set_offset( self.handle, value )

	def get_time_constant( self ):
		return MpPrivate.amplifier_get_time_constant( self.handle )

	def set_time_constant( self, value ):
		MpPrivate.amplifier_set_time_constant( self.handle, value )

#
#  Analog input class
#

class AnalogInput(Record):

	def read( self ):
		return MpPrivate.analog_input_read( self.handle )

	def read_raw_long( self ):
		return MpPrivate.analog_input_read_raw_long( self.handle )

	def read_raw_double( self ):
		return MpPrivate.analog_input_read_raw_double( self.handle )

	def get_dark_current( self ):
		return MpPrivate.analog_input_get_dark_current( self.handle )

	def set_dark_current( self, dark_current ):
		MpPrivate.analog_input_set_dark_current( self.handle, \
							dark_current )

#
#  Analog output class
#

class AnalogOutput(Record):

	def read( self ):
		return MpPrivate.analog_output_read( self.handle )

	def read_raw_long( self ):
		return MpPrivate.analog_output_read_raw_long( self.handle )

	def read_raw_double( self ):
		return MpPrivate.analog_output_read_raw_double( self.handle )

	def write( self, value ):
		MpPrivate.analog_output_write( self.handle, value )

	def write_raw_long( self, value ):
		MpPrivate.analog_output_write_raw_long( self.handle, value )

	def write_raw_double( self, value ):
		MpPrivate.analog_output_write_raw_double( self.handle, value )

#
#  Autoscale class
#

class Autoscale(Record):
	def read_monitor( self ):
		return MpPrivate.autoscale_read_monitor( self.handle )

	def get_change_request( self ):
		return MpPrivate.autoscale_get_change_request( self.handle )

	def change_control( self, change_control ):
		MpPrivate.autoscale_change_control( self.handle, \
							change_control )

	def get_offset_index( self ):
		return MpPrivate.autoscale_get_offset_index( self.handle )

	def set_offset_index( self, monitor_offset_index ):
		MpPrivate.autoscale_set_offset_index( self.handle, \
							monitor_offset_index )

	def get_offsets( self ):
		return MpPrivate.autoscale_get_offsets( self.handle )
					
	def set_offsets( self, monitor_offsets ):
		MpPrivate.autoscale_set_offsets( self.handle, monitor_offsets )

	def get_limits( self ):
		return MpPrivate.autoscale_get_limits( self.handle )

	def set_limits( self, limits_tuple ):
		"""
		limits_tuple includes 
		( low_limit, high_limit, low_deadband, high_deadband )
		"""
		MpPrivate.autoscale_set_limits( self.handle, limits_tuple )

	def compute_dynamic_limits( self ):
		MpPrivate.autoscale_compute_dynamic_limits( self.handle )

#
#  CCD class
#

class CCD(Record):

	def start( self ):
		MpPrivate.ccd_start( self.handle )

	def start_for_preset_time( self, time_in_seconds ):
		MpPrivate.ccd_start_for_preset_time( self.handle, \
							time_in_seconds )

	def stop( self ):
		MpPrivate.ccd_stop( self.handle )

	def get_status( self ):
		return MpPrivate.ccd_get_status( self.handle )

	def readout( self, flags ):
		MpPrivate.ccd_readout( self.handle, flags )

	def dezinger( self, flags ):
		MpPrivate.ccd_dezinger( self.handle, flags )

	def correct( self ):
		MpPrivate.ccd_correct( self.handle )

	def writefile( self, filename, flags ):
		MpPrivate.ccd_writefile( self.handle, filename, flags )

	def get_data_frame_size( self ):
		return MpPrivate.ccd_get_data_frame_size( self.handle )

	def get_bin_size( self ):
		return MpPrivate.ccd_get_bin_size( self.handle )

	def set_bin_size( self, bin_size_tuple ):
		MpPrivate.ccd_set_bin_size( self.handle, bin_size_tuple )

	def write_header_variable( self, variable_name, variable_contents ):
		MpPrivate.ccd_write_header_variable( self.handle, \
					variable_name, variable_contents )

#
#  Digital input class
#

class DigitalInput(Record):

	def read( self ):
		return MpPrivate.digital_input_read( self.handle )

#
#  Digital output class
#

class DigitalOutput(Record):

	def read( self ):
		return MpPrivate.digital_output_read( self.handle )

	def write( self, value ):
		MpPrivate.digital_output_write( self.handle, value )

#
#  Encoder class
#

class Encoder(Record):
	def get_overflow_status( self ):
		return MpPrivate.encoder_get_overflow_status( self.handle )

	def reset_overflow_status( self ):
		MpPrivate.encoder_reset_overflow_status( self.handle )

	def read( self ):
		return MpPrivate.encoder_read( self.handle )

	def write( self, value ):
		return MpPrivate.encoder_write( self.handle, value )

#
#  Multichannel analyzer class
#

class MCA(Record):
	def start( self ):
		MpPrivate.mca_start( self.handle )

	def stop( self ):
		MpPrivate.mca_stop( self.handle )

	def read( self ):
		return MpPrivate.mca_read( self.handle )

	def clear( self ):
		MpPrivate.mca_clear( self.handle )

	def is_busy( self ):
		return MpPrivate.mca_is_busy( self.handle )

	def start_without_preset( self ):
		MpPrivate.mca_start_without_preset( self.handle )

	def start_with_preset( self, preset_type, preset_value ):
		MpPrivate.mca_start_without_preset( self.handle, \
						preset_type, preset_value )

	def start_for_preset_live_time( self, preset_live_time ):
		MpPrivate.mca_start_for_preset_live_time( self.handle, \
							preset_live_time )

	def start_for_preset_real_time( self, preset_real_time ):
		MpPrivate.mca_start_for_preset_real_time( self.handle, \
							preset_real_time )

	def start_for_preset_count( self, preset_count ):
		MpPrivate.mca_start_for_preset_count( self.handle, \
							preset_count )

	def get_preset_type( self ):
		return MpPrivate.mca_get_preset_type( self.handle )

	def set_preset_type( self, preset_type ):
		MpPrivate.mca_set_preset_type( self.handle, preset_type )

	def get_preset_count_region( self ):
		return MpPrivate.mca_get_preset_count_region( self.handle )

	def set_preset_count_region( self, preset_count_region ):
		MpPrivate.mca_set_preset_count_region( self.handle, \
							preset_count_region )

	def get_roi( self, roi_number ):
		return MpPrivate.mca_get_roi( self.handle, roi_number )

	def set_roi( self, roi_number, roi_tuple ):
		MpPrivate.mca_set_roi( self.handle, roi_number, roi_tuple )

	def get_roi_integral( self, roi_number ):
		return MpPrivate.mca_get_roi_integral( self.handle, roi_number )

	def get_rois( self ):
		return MpPrivate.mca_get_rois( self.handle )

	def set_rois( self, rois_tuple ):
		MpPrivate.mca_set_rois( self.handle, rois_tuple )

	def get_roi_integrals( self ):
		return MpPrivate.mca_get_roi_integrals( self.handle, \
							 roi_integrals_tuple )

	def get_num_channels( self ):
		return MpPrivate.mca_get_num_channels( self.handle )

	def set_num_channels( self, num_channels ):
		MpPrivate.mca_set_num_channels( self.handle, num_channels )

	def get_channel( self ):
		return MpPrivate.mca_get_channel( self.handle, channel_number )

	def get_real_time( self ):
		return MpPrivate.mca_get_real_time( self.handle )

	def get_live_time( self ):
		return MpPrivate.mca_get_live_time( self.handle )

	def get_counts( self ):
		return MpPrivate.mca_get_counts( self.handle )

	def get_soft_roi( self, soft_roi_number ):
		return MpPrivate.mca_get_soft_roi( self.handle, \
							soft_roi_number )

	def set_soft_roi( self, soft_roi_number, soft_roi_tuple ):
		MpPrivate.mca_set_soft_roi( self.handle, \
					 soft_roi_number, soft_roi_tuple )

	def get_soft_integral( self, soft_roi_number ):
		return MpPrivate.mca_get_soft_roi_integral( self.handle, \
							 soft_roi_number )

#
#  Multichannel encoder class
#

class MCE(Record):
	def get_overflow_status( self ):
		return MpPrivate.mce_get_overflow_status( self.handle )

	def reset_overflow_status( self ):
		MpPrivate.mce_reset_overflow_status( self.handle )

	def read( self ):
		return MpPrivate.mce_read( self.handle )

	def get_current_num_values( self ):
		return MpPrivate.mce_get_current_num_values( self.handle )

#
#  Multichannel scaler class
#

class MCS(Record):
	def start( self ):
		MpPrivate.mcs_start( self.handle )

	def stop( self ):
		MpPrivate.mcs_stop( self.handle )

	def clear( self ):
		MpPrivate.mcs_clear( self.handle )

	def is_busy( self ):
		""" Returns busy flag """
		return MpPrivate.mcs_is_busy( self.handle )

	def read_all( self ):
		"""
		Returns a 2-D mcs_data tuple.
		"""
		return MpPrivate.mcs_read_all( self.handle )

	def read_scaler( self, scaler_index ):
		"""
		Returns a 1-D scaler_data tuple.
		"""
		return MpPrivate.mcs_read_scaler( self.handle, scaler_index )

	def read_measurement( self, measurement_index ):
		"""
		Returns a 1-D measurement_data tuple.
		"""
		return MpPrivate.mcs_read_measurement( self.handle, \
							measurement_index )

	def read_timer( self ):
		"""
		Returns a 1-D timer_data tuple.
		"""
		return MpPrivate.mcs_read_timer( self.handle )

	def get_mode( self ):
		""" Returns the counter mode setting """
		return MpPrivate.mcs_get_mode( self.handle )

	def set_mode( self, mode ):
		MpPrivate.mcs_set_mode( self.handle, mode )

	def get_measurement_time( self ):
		""" Returns measurement time per point """
		return MpPrivate.mcs_get_measurement_time( self.handle )

	def set_measurement_time( self, measurement_time ):
		MpPrivate.mcs_set_measurement_time( self.handle, \
							measurement_time )

	def get_measurement_counts( self ):
		""" Returns measurement counts per point """
		return MpPrivate.mcs_get_measurement_counts( self.handle )

	def set_measurement_counts( self, measurement_counts ):
		MpPrivate.mcs_set_measurement_counts( self.handle, \
							measurement_counts )

	def get_num_measurements( self ):
		""" Returns the number of measurements """
		return MpPrivate.mcs_get_num_measurements( self.handle )

	def set_num_measurements( self, num_measurements ):
		MpPrivate.mcs_set_num_measurements( self.handle, \
							num_measurements )

#
#  Motor class
#

class Motor(Record):

	tweak_distance = 0.0

	def is_busy( self ):
		return MpPrivate.motor_is_busy( self.handle )

	def move_absolute( self, new_destination ):
		MpPrivate.motor_move_absolute( self.handle, new_destination )

	def move_relative( self, relative_position ):
		MpPrivate.motor_move_relative( self.handle, relative_position )

	def move_absolute_with_flags( self, new_destination, flags ):
		MpPrivate.motor_move_absolute_with_flags( self.handle, \
						new_destination, flags )

	def move_relative_with_flags( self, relative_position, flags ):
		MpPrivate.motor_move_relative_with_flags( self.handle, \
						relative_position, flags )

	def wait_for_motor_stop( self, flags ):
		MpPrivate.motor_wait_for_motor_stop( self.handle, flags )

	def get_position( self ):
		return MpPrivate.motor_get_position( self.handle )

	def set_position( self, new_set_position ):
		MpPrivate.motor_set_position( self.handle, new_set_position )

	def soft_abort( self ):
		MpPrivate.motor_soft_abort( self.handle )

	def immediate_abort( self ):
		MpPrivate.motor_immediate_abort( self.handle )

	def positive_limit_hit( self ):
		return MpPrivate.motor_positive_limit_hit( self.handle )

	def negative_limit_hit( self ):
		return MpPrivate.motor_negative_limit_hit( self.handle )

	def find_home_position( self, direction ):
		MpPrivate.motor_find_home_position( self.handle, direction )

	def constant_velocity_move( self, direction ):
		MpPrivate.motor_constant_velocity_move( self.handle, direction )

	def get_speed( self ):
		return MpPrivate.motor_get_speed( self.handle )

	def set_speed( self, speed ):
		MpPrivate.motor_set_speed( self.handle, speed )

	def get_base_speed( self ):
		return MpPrivate.motor_get_base_speed( self.handle )

	def set_base_speed( self, base_speed ):
		MpPrivate.motor_set_base_speed( self.handle, base_speed )

	def get_maximum_speed( self ):
		return MpPrivate.motor_get_maximum_speed( self.handle )

	def set_maximum_speed( self, maximum_speed ):
		MpPrivate.motor_set_maximum_speed( self.handle, maximum_speed )

	def save_speed( self ):
		MpPrivate.motor_save_speed( self.handle )

	def restore_speed( self ):
		MpPrivate.motor_restore_speed( self.handle )

	def set_speed_between_positions( self, \
			position1, position2, time_for_move ):
		MpPrivate.motor_set_speed_between_positions( self.handle, \
			position1, position2, time_for_move )

	def get_acceleration_type( self ):
		return MpPrivate.motor_get_acceleration_type( self.handle )

	def get_raw_acceleration_parameters( self ):
		return MpPrivate.motor_get_raw_acceleration_parameters( \
								self.handle )

	def set_raw_acceleration_parameters( self, parameter_list ):
		MpPrivate.motor_set_raw_acceleration_parameters( self.handle, \
								parameter_list )

	def get_acceleration_time( self ):
		return MpPrivate.motor_get_acceleration_time( self.handle )

	def set_acceleration_time( self ):
		MpPrivate.motor_set_acceleration_time( self.handle, \
							acceleration_time )

	def get_acceleration_distance( self ):
		return MpPrivate.motor_get_acceleration_distance( self.handle )

	def home_search_succeeded( self ):
		MpPrivate.motor_home_search_succeeded( self.handle )

	def get_gain( self, gain_type ):
		return MpPrivate.motor_get_gain( self.handle, gain_type )

	def set_gain( self, gain_type, gain ):
		MpPrivate.motor_set_gain( self.handle, gain_type, gain )

	def send_control_command( self ):
		MpPrivate.motor_send_control_command( self.handle,
						command_type, command )

#
#  Functions that operate on multiple motor objects.
#
def multiple_motors_move_absolute( motors_tuple, positions_tuple ):
	MpPrivate.multiple_motors_move_absolute( motors_tuple, positions_tuple )

def wait_for_multiple_motors_stop( motors_tuple ):
	MpPrivate.wait_for_multiple_motors_stop( motors_tuple )

#
#  Pulse Generator class
#

class PulseGenerator(Record):
	def is_busy( self ):
		return MpPrivate.pulse_generator_is_busy( self.handle )

	def start( self ):
		MpPrivate.pulse_generator_start( self.handle )

	def stop( self ):
		MpPrivate.pulse_generator_stop( self.handle )

	def get_mode( self ):
		return MpPrivate.pulse_generator_get_mode( self.handle )

	def set_mode( self, mode ):
		MpPrivate.pulse_generator_set_mode( self.handle, mode )

	def get_pulse_period( self ):
		return MpPrivate.pulse_generator_get_pulse_period( self.handle )

	def set_pulse_period( self, pulse_period ):
		MpPrivate.pulse_generator_set_pulse_period( self.handle,
								pulse_period )

	def get_pulse_width( self ):
		return MpPrivate.pulse_generator_get_pulse_width( self.handle )

	def set_pulse_width( self, pulse_width ):
		MpPrivate.pulse_generator_set_pulse_width( self.handle,
								pulse_width )

	def get_num_pulses( self ):
		return MpPrivate.pulse_generator_get_num_pulses( self.handle )

	def set_num_pulses( self, num_pulses ):
		MpPrivate.pulse_generator_set_num_pulses( self.handle,
								num_pulses )

	def get_pulse_delay( self ):
		return MpPrivate.pulse_generator_get_pulse_delay( self.handle )

	def set_pulse_delay( self, pulse_delay ):
		MpPrivate.pulse_generator_set_pulse_delay( self.handle,
								pulse_delay )


#
#  Relay class
#

class Relay(Record):
	def relay_command( self, command ):
		MpPrivate.relay_relay_command( self.handle, command )

	def get_relay_status( self ):
		return MpPrivate.relay_get_relay_status( self.handle )

#
#  Sample changer class
#

class SampleChanger(Record):
	def initialize( self ):
		MpPrivate.sample_changer_initialize( self.handle )

	def shutdown( self ):
		MpPrivate.sample_changer_shutdown( self.handle )

	def select_sample_holder( self, sample_holder_name ):
		MpPrivate.sample_changer_select_sample_holder( self.handle,
							   sample_holder_name )

	def unselect_sample_holder( self ):
		MpPrivate.sample_changer_unselect_sample_holder( self.handle )

	def grab_sample( self, sample_id ):
		MpPrivate.sample_changer_grab_sample( self.handle, sample_id )

	def ungrab_sample( self ):
		MpPrivate.sample_changer_ungrab_sample( self.handle )

	def mount_sample( self ):
		MpPrivate.sample_changer_mount_sample( self.handle )

	def unmount_sample( self ):
		MpPrivate.sample_changer_unmount_sample( self.handle )

	def get_sample_id( self ):
		return MpPrivate.sample_changer_get_sample_id( self.handle )

	def set_sample_id( self, sample_id ):
		MpPrivate.sample_changer_set_sample_id( self.handle, sample_id )

	def soft_abort( self ):
		MpPrivate.sample_changer_soft_abort( self.handle )

	def immediate_abort( self ):
		MpPrivate.sample_changer_immediate_abort( self.handle )

	def idle( self ):
		MpPrivate.sample_changer_idle( self.handle )

	def reset( self ):
		MpPrivate.sample_changer_reset( self.handle )

	def get_status( self ):
		return MpPrivate.sample_changer_get_status( self.handle )

	def is_busy( self ):
		return MpPrivate.sample_changer_is_busy( self.handle )

	def cooldown( self ):
		MpPrivate.sample_changer_cooldown( self.handle )

	def deice( self ):
		MpPrivate.sample_changer_deice( self.handle )

#
#  Single channel analyzer class
#

class SCA(Record):
	def get_lower_level( self ):
		return MpPrivate.sca_get_lower_level( self.handle )

	def set_lower_level( self, lower_level ):
		MpPrivate.sca_set_lower_level( self.handle, lower_level )

	def get_upper_level( self ):
		return MpPrivate.sca_get_upper_level( self.handle )

	def set_upper_level( self, upper_level ):
		MpPrivate.sca_set_upper_level( self.handle, upper_level )

	def get_gain( self ):
		MpPrivate.sca_get_gain( self.handle )

	def set_gain( self, gain ):
		MpPrivate.sca_set_gain( self.handle, gain )

	def get_time_constant( self ):
		MpPrivate.sca_get_time_constant( self.handle )

	def set_time_constant( self, time_constant ):
		MpPrivate.sca_set_time_constant( self.handle, time_constant )

	def get_mode( self ):
		MpPrivate.sca_get_mode( self.handle )

	def set_mode( self, mode ):
		MpPrivate.sca_set_mode( self.handle, mode )

#
#  Scaler class
#

class Scaler(Record):
	def clear( self ):
		MpPrivate.scaler_clear( self.handle )

	def overflow_set( self ):
		return MpPrivate.scaler_overflow_set( self.handle )

	def read( self ):
		return MpPrivate.scaler_read( self.handle )

	def read_raw( self ):
		return MpPrivate.scaler_read_raw( self.handle )

	def is_busy( self ):
		return MpPrivate.scaler_is_busy( self.handle )

	def start( self, preset_count ):
		MpPrivate.scaler_start( self.handle, preset_count )

	def stop( self ):
		return MpPrivate.scaler_stop( self.handle )

	def get_mode( self ):
		return MpPrivate.scaler_get_mode( self.handle )

	def set_mode( self, mode ):
		MpPrivate.scaler_set_mode( self.handle, mode )

	def set_modes_of_associated_counters( self ):
		MpPrivate.scaler_set_modes_of_associated_counters( self.handle )

	def get_dark_current( self ):
		return MpPrivate.scaler_get_dark_current( self.handle )

	def set_dark_current( self, dark_current ):
		MpPrivate.scaler_set_dark_current( self.handle, dark_current )

	def read_dark_current_subtracted_value( self, timer ):
		return MpPrivate.scaler_read_dark_current_subtracted_value( \
						self.handle, timer )

#
#  Table class
#

class Table(Record):

	def is_busy( self, axis_id ):
		return MpPrivate.table_is_busy( self.handle, axis_id )

	def move_absolute( self, axis_id, new_destination ):
		MpPrivate.table_move_absolute( self.handle, \
						axis_id, new_destination )

	def get_position( self, axis_id ):
		return MpPrivate.table_get_position( self.handle, axis_id )

	def set_position( self, axis_id, new_set_position ):
		MpPrivate.table_set_position( self.handle, \
						axis_id, new_set_position )

	def soft_abort( self, axis_id ):
		MpPrivate.table_soft_abort( self.handle, axis_id )

	def immediate_abort( self, axis_id ):
		MpPrivate.table_immediate_abort( self.handle, axis_id )

	def positive_limit_hit( self, axis_id ):
		return MpPrivate.table_positive_limit_hit( \
						self.handle, axis_id )

	def negative_limit_hit( self, axis_id ):
		return MpPrivate.table_negative_limit_hit( \
						self.handle, axis_id )

#
#  Timer class
#

class Timer(Record):
	def is_busy( self ):
		return MpPrivate.timer_is_busy( self.handle )

	def start( self, seconds ):
		MpPrivate.timer_start( self.handle, seconds )

	def stop( self ):
		return MpPrivate.timer_stop( self.handle )

	def clear( self ):
		MpPrivate.timer_clear( self.handle )

	def read( self ):
		return MpPrivate.timer_read( self.handle )

	def get_mode( self ):
		return MpPrivate.timer_get_mode( self.handle )

	def set_mode( self, mode ):
		MpPrivate.timer_set_mode( self.handle, mode )

	def set_modes_of_associated_counters( self ):
		MpPrivate.timer_set_modes_of_associated_counters( self.handle )

	def get_last_measurement_time( self ):
		return MpPrivate.timer_get_last_measurement_time( self.handle )

#
#  Scan class
#

class Scan(Record):
	def perform_scan( self ):
		MpPrivate.scan_perform_scan( self.handle )

#
#  Variable class
#

class Variable(Record):
	def read( self ):
		return MpPrivate.variable_receive_variable( self.handle )

	def write( self, value ):
		MpPrivate.variable_send_variable( self.handle, value )

	def receive_variable( self ):
		return MpPrivate.variable_receive_variable( self.handle )

	def send_variable( self, value ):
		MpPrivate.variable_send_variable( self.handle, value )

