/*
 * Name:    mp_rs232.c
 *
 * Purpose: Python module interface to generic MX RS-232 functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002-2003 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_rs232.h"
#include "mx_handle.h"
#include "mp.h"

#include "mx_key.h"

MX_EXPORT PyObject *
MpPrivate_rs232_getchar_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	char c;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_getchar( record, &c, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the character back to Python as a string object. */

	result = Py_BuildValue( "b", c );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_rs232_putchar_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	char c;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lb", &handle, &c );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the character to the RS-232 port. */

	mx_status = mx_rs232_putchar( record, c, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_rs232_read_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "read";

	PyObject *result;
	MX_RECORD *record;
	int parse_status;
	long handle, num_bytes_to_read;
	size_t bytes_read;
	char *buffer_ptr;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll",
					&handle, &num_bytes_to_read );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	buffer_ptr = (char *) malloc( num_bytes_to_read * sizeof(char) );

	if ( buffer_ptr == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element buffer.",
				cname, num_bytes_to_read );
		return NULL;
	}

	/* Read into the buffer from the RS-232 port. */

	mx_status = mx_rs232_read( record, buffer_ptr,
				(size_t) num_bytes_to_read, &bytes_read, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "s#", buffer_ptr, (int) bytes_read );

	mx_free( buffer_ptr );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_rs232_write_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "write";

	MX_RECORD *record;
	int parse_status, num_bytes_to_write;
	long handle;
	size_t bytes_written;
	char *buffer;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ls#",
					&handle, &buffer, &num_bytes_to_write );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the buffer to the RS-232 port. */

	mx_status = mx_rs232_write( record, buffer,
			(size_t) num_bytes_to_write, &bytes_written, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	if ( (int) bytes_written != num_bytes_to_write ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
		"%s: The number of bytes written to RS-232 port '%s', %ld, "
		"does not match the number of bytes that were supposed "
		"to be written, %ld.", cname, record->name,
			(long) bytes_written, (long) num_bytes_to_write );
		return NULL;
	}

	Py_INCREF(Py_None);

	return Py_None;
}

#define MPPRIVATE_RS232_DEFAULT_BUFFER_SIZE	1024

MX_EXPORT PyObject *
MpPrivate_rs232_getline_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "getline";

	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	long maximum_length;
	char local_buffer[ MPPRIVATE_RS232_DEFAULT_BUFFER_SIZE + 1 ];
	char *malloc_buffer;
	char *buffer_ptr;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &maximum_length );

	if ( parse_status == 0 )
		return NULL;

	if ( maximum_length == MPPRIVATE_RS232_DEFAULT_BUFFER_SIZE ) {
		malloc_buffer = NULL;
		buffer_ptr = local_buffer;
	} else {
		malloc_buffer = (char *)
				malloc( maximum_length * sizeof(char) );

		if ( malloc_buffer == NULL ) {
			MpPrivateErr_SetString( PyExc_MemoryError,
	"%s: Ran out of memory trying to allocate a %ld element buffer.",
				cname, maximum_length );
			return NULL;
		}
		buffer_ptr = malloc_buffer;
	}

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_getline( record, buffer_ptr,
					(size_t) maximum_length, NULL, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the line back to Python as a string object. */

	result = Py_BuildValue( "s", buffer_ptr );

	mx_free( malloc_buffer );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_rs232_putline_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	char *buffer;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ls", &handle, &buffer );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Send the line to the RS-232 port. */

	mx_status = mx_rs232_putline( record, buffer, NULL, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_rs232_input_is_available_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, input_available;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_input_is_available( record, &input_available );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", input_available );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_rs232_discard_unread_input_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_discard_unread_input( record, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_rs232_discard_unwritten_output_cmd(PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_discard_unwritten_output( record, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_rs232_get_signal_state_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long signal_state;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_get_signal_state( record, &signal_state );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "l", signal_state );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_rs232_set_signal_state_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	unsigned long signal_state;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &signal_state );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_rs232_set_signal_state( record, signal_state );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

